/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2012 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.imap.entity2acl;

import com.openexchange.exception.Category;
import com.openexchange.exception.OXException;
import com.openexchange.exception.OXExceptionCode;
import com.openexchange.exception.OXExceptionFactory;

public enum Entity2ACLExceptionCode implements OXExceptionCode {

    /**
     * Implementing class could not be found
     */
    CLASS_NOT_FOUND(Entity2ACLExceptionMessage.CLASS_NOT_FOUND_MSG, Category.CATEGORY_ERROR, 1),
    /**
     * An I/O error occurred while creating the socket connection to IMAP server (%1$s): %2$s
     */
    CREATING_SOCKET_FAILED(Entity2ACLExceptionMessage.CREATING_SOCKET_FAILED_MSG, Category.CATEGORY_SERVICE_DOWN, 2),
    /**
     * Instantiating the class failed.
     */
    INSTANTIATION_FAILED(Entity2ACLExceptionMessage.INSTANTIATION_FAILED_MSG, Category.CATEGORY_ERROR, 3),
    /**
     * An I/O error occurred: %1$s
     */
    IO_ERROR(Entity2ACLExceptionMessage.IO_ERROR_MSG, Category.CATEGORY_SERVICE_DOWN, 4),
    /**
     * Missing property %1$s in system.properties.
     */
    MISSING_SETTING(Entity2ACLExceptionMessage.MISSING_SETTING_MSG, Category.CATEGORY_CONFIGURATION, 5),
    /**
     * Unknown IMAP server: %1$s
     */
    UNKNOWN_IMAP_SERVER(Entity2ACLExceptionMessage.UNKNOWN_IMAP_SERVER_MSG, Category.CATEGORY_ERROR, 6),
    /**
     * Missing IMAP server arguments to resolve IMAP login to a user
     */
    MISSING_ARG(Entity2ACLExceptionMessage.MISSING_ARG_MSG, Category.CATEGORY_ERROR, 7),
    /**
     * IMAP login %1$s could not be resolved to a user
     */
    RESOLVE_USER_FAILED(Entity2ACLExceptionMessage.RESOLVE_USER_FAILED_MSG, Category.CATEGORY_ERROR, 8),
    /**
     * User %1$s from context %2$s is not known on IMAP server "%3$s".
     */
    UNKNOWN_USER(Entity2ACLExceptionMessage.UNKNOWN_USER_MSG, Category.CATEGORY_ERROR, 9);

    /**
     * Category of the exception.
     */
    final Category category;

    /**
     * Message of the exception.
     */
    final String message;

    /**
     * Detail number of the exception.
     */
    final int number;

    /**
     * Default constructor.
     *
     * @param message message.
     * @param category category.
     * @param detailNumber detail number.
     */
    private Entity2ACLExceptionCode(final String message, final Category category, final int detailNumber) {
        this.message = message;
        this.category = category;
        number = detailNumber;
    }

    @Override
    public String getPrefix() {
        return "ACL";
    }

    /**
     * @return the category.
     */
    @Override
    public Category getCategory() {
        return category;
    }

    /**
     * @return the message.
     */
    @Override
    public String getMessage() {
        return message;
    }

    /**
     * @return the number.
     */
    @Override
    public int getNumber() {
        return number;
    }

    @Override
    public boolean equals(final OXException e) {
        return OXExceptionFactory.getInstance().equals(this, e);
    }

    /**
     * Creates a new {@link OXException} instance pre-filled with this code's attributes.
     *
     * @return The newly created {@link OXException} instance
     */
    public OXException create() {
        return OXExceptionFactory.getInstance().create(this, new Object[0]);
    }

    /**
     * Creates a new {@link OXException} instance pre-filled with this code's attributes.
     *
     * @param args The message arguments in case of printf-style message
     * @return The newly created {@link OXException} instance
     */
    public OXException create(final Object... args) {
        return OXExceptionFactory.getInstance().create(this, (Throwable) null, args);
    }

    /**
     * Creates a new {@link OXException} instance pre-filled with this code's attributes.
     *
     * @param cause The optional initial cause
     * @param args The message arguments in case of printf-style message
     * @return The newly created {@link OXException} instance
     */
    public OXException create(final Throwable cause, final Object... args) {
        return OXExceptionFactory.getInstance().create(this, cause, args);
    }
}
